@@ -5,13 +5,13 @@ use std::sync::Arc;
 use tokio::task::JoinHandle;
 
 mod protocol;
-use crate::config::BaseConfig;
+use crate::config::{ParsedConfig, Upstream};
 use protocol::{kcp, tcp};
 
 #[derive(Debug)]
 pub struct Server {
     pub proxies: Vec<Arc<Proxy>>,
-    pub config: BaseConfig,
+    pub config: ParsedConfig,
 }
 
 #[derive(Debug, Clone)]
@@ -22,11 +22,11 @@ pub struct Proxy {
     pub tls: bool,
     pub sni: Option<HashMap<String, String>>,
     pub default: String,
-    pub upstream: HashMap<String, String>,
+    pub upstream: HashMap<String, Upstream>,
 }
 
 impl Server {
-    pub fn new(config: BaseConfig) -> Self {
+    pub fn new(config: ParsedConfig) -> Self {
         let mut new_server = Server {
             proxies: Vec::new(),
             config: config.clone(),
@@ -53,6 +53,7 @@ impl Server {
                         continue;
                     }
                 };
+
                 let proxy = Proxy {
                     name: name.clone(),
                     listen: listen_addr,
@@ -103,7 +104,7 @@ impl Server {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use crate::plugins::kcp::{KcpConfig, KcpStream};
     use std::net::SocketAddr;
     use std::thread::{self, sleep};
@@ -117,13 +118,24 @@ mod test {
     async fn tcp_mock_server() {
         let server_addr: SocketAddr = "127.0.0.1:54599".parse().unwrap();
         let listener = TcpListener::bind(server_addr).await.unwrap();
-        let (mut stream, _) = listener.accept().await.unwrap();
-        stream.write(b"hello").await.unwrap();
-        stream.shutdown().await.unwrap();
+        loop {
+            let (mut stream, _) = listener.accept().await.unwrap();
+            let mut buf = [0u8; 2];
+            let mut n = stream.read(&mut buf).await.unwrap();
+            while n > 0 {
+                stream.write(b"hello").await.unwrap();
+                if buf.eq(b"by") {
+                    stream.shutdown().await.unwrap();
+                    break;
+                }
+                n = stream.read(&mut buf).await.unwrap();
+            }
+            stream.shutdown().await.unwrap();
+        }
     }
 
     #[tokio::test]
-    async fn test_tcp_proxy() {
+    async fn test_proxy() {
         use crate::config::Config;
         let config = Config::new("tests/config.yaml").unwrap();
         let mut server = Server::new(config.base);
@@ -135,41 +147,26 @@ mod test {
             let _ = server.run();
         });
         sleep(Duration::from_secs(1)); // wait for server to start
+
+        // test TCP proxy
         let mut conn = TcpStream::connect("127.0.0.1:54500").await.unwrap();
         let mut buf = [0u8; 5];
+        conn.write(b"hi").await.unwrap();
         conn.read(&mut buf).await.unwrap();
         assert_eq!(&buf, b"hello");
         conn.shutdown().await.unwrap();
-    }
 
-    #[tokio::test]
-    async fn test_tcp_echo_server() {
-        use crate::config::Config;
-        let config = Config::new("tests/config.yaml").unwrap();
-        let mut server = Server::new(config.base);
-        thread::spawn(move || {
-            let _ = server.run();
-        });
-        sleep(Duration::from_secs(1)); // wait for server to start
+        // test TCP echo
         let mut conn = TcpStream::connect("127.0.0.1:54956").await.unwrap();
         let mut buf = [0u8; 1];
-        for i in 0..=255u8 {
+        for i in 0..=10u8 {
             conn.write(&[i]).await.unwrap();
             conn.read(&mut buf).await.unwrap();
             assert_eq!(&buf, &[i]);
         }
         conn.shutdown().await.unwrap();
-    }
 
-    #[tokio::test]
-    async fn test_kcp_echo_server() {
-        use crate::config::Config;
-        let config = Config::new("tests/config.yaml").unwrap();
-        let mut server = Server::new(config.base);
-        thread::spawn(move || {
-            let _ = server.run();
-        });
-        sleep(Duration::from_secs(1)); // wait for server to start
+        // test KCP echo
         let kcp_config = KcpConfig::default();
         let server_addr: SocketAddr = "127.0.0.1:54959".parse().unwrap();
         let mut conn = KcpStream::connect(&kcp_config, server_addr).await.unwrap();
@@ -180,5 +177,15 @@ mod test {
             assert_eq!(&buf, &[i]);
         }
         conn.shutdown().await.unwrap();
+
+        // test KCP proxy and close mock server
+        let kcp_config = KcpConfig::default();
+        let server_addr: SocketAddr = "127.0.0.1:54958".parse().unwrap();
+        let mut conn = KcpStream::connect(&kcp_config, server_addr).await.unwrap();
+        let mut buf = [0u8; 5];
+        conn.write(b"by").await.unwrap();
+        conn.read(&mut buf).await.unwrap();
+        assert_eq!(&buf, b"hello");
+        conn.shutdown().await.unwrap();
     }
 }
